#!/usr/bin/env python
from python_crypto.customcrypto import dir_encrypt, dir_decrypt
import sys
from getpass import getpass

DEFAULT_DIR = '.secrets'

def run_command():
    command = sys.argv[1].lower()
    dir = sys.argv[2].lower() if len(sys.argv) >= 3 else DEFAULT_DIR
    if command == 'decrypt':
        password = getpass()
        dir_decrypt(password, dir)
    elif command == 'encrypt':
        password = getpass()
        dir_encrypt(password, dir)
    else:
        print("bad command.")

if __name__ == "__main__":
    run_command()


import hashlib

from AesEverywhere import aes256
from aws_encryption_sdk.exceptions import DecryptKeyError
from python_crypto.awscrypto import encrypt, decrypt, FAKE_ENCRYPTION
from shutil import copyfile, copytree, rmtree
import os


def hash_value(v):
    return hashlib.sha224(bytes(v, 'utf-8')).hexdigest()

def password_encrypt(plaintext, password):
    safe_pass = password.strip()
    return aes256.encrypt(plaintext, safe_pass)

def password_decrypt(ciphertext, password):
    safe_pass = password.strip()
    return aes256.decrypt(ciphertext, safe_pass).decode("utf-8")

def _replace_in_bytes(byte_str, old, new):
    start_ind = byte_str.find(old)
    return byte_str[:start_ind] + new + byte_str[start_ind+len(old):]

def double_encrypt(plaintext, password, context={}):
    if FAKE_ENCRYPTION:
        return password_encrypt(plaintext, password)
    safe_pass = password.strip()
    hashed_pass = hash_value(safe_pass)
    aws_cipher = encrypt(plaintext, encryption_context={
        'pass': hashed_pass,
        **context,
    })
    stripped_cipher = _replace_in_bytes(aws_cipher, bytes(hashed_pass, 'utf-8'), b'{PASS}')
    return stripped_cipher

class BadEncryptionPassword(Exception):
    pass

def double_decrypt(stripped_ciphertext, password):
    if FAKE_ENCRYPTION:
        return password_decrypt(stripped_ciphertext, password)
    safe_pass = password.strip()
    hashed_pass = hash_value(safe_pass)
    ciphertext = _replace_in_bytes(stripped_ciphertext, b'{PASS}', bytes(hashed_pass, 'utf-8'))
    try:
        plaintext = decrypt(ciphertext)
    except DecryptKeyError as e:
        raise BadEncryptionPassword
    return plaintext


def _get_encryption_method(method_name):
    method_name = method_name.lower()
    if method_name == "double_encrypt":
        return double_encrypt
    elif method_name == "double_decrypt":
        return double_decrypt
    elif method_name == "password_encrypt":
        return password_encrypt
    elif method_name == "password_decrypt":
        return password_decrypt
    elif method_name == "aws_encrypt":
        return encrypt
    elif method_name == "aws_decrypt":
        return decrypt

def file_encrypt(from_path, to_path, password=None, method="double_encrypt", **kwargs):
    method = _get_encryption_method(method)
    from_file = open(from_path, 'r')
    lines = ''.join(from_file.readlines())
    from_file.close()
    if password:
        ciphertext = method(lines, password, **kwargs)
    else:
        ciphertext = method(lines, **kwargs)
    to_file = open(to_path, 'wb')
    to_file.write(ciphertext)
    to_file.close()

def file_decrypt(from_path, to_path, password=None, method="double_decrypt", **kwargs):
    method = _get_encryption_method(method)
    from_file = open(from_path, 'rb')
    ciphertext = from_file.read()
    from_file.close()
    if password:
        plaintext = method(ciphertext, password, **kwargs)
    else:
        plaintext = method(ciphertext, **kwargs)
    to_file = open(to_path, 'w')
    to_file.write(plaintext)
    to_file.close()



def dir_encrypt(password, dir_name, **kwargs):
    status_file = f'{dir_name}/.status'
    state = "decrypted"
    try:
        status = open(status_file, "r")
        state = status.readline().strip()
        status.close()
    except:
        pass
    if state == "decrypted":
        status = open(status_file, "w")
        status.write("encrypted\n")
        status.close()
        backup_dir = f'{dir_name}.backup'
        copytree(dir_name, backup_dir)
        try:
            for dirpath, subdirs, files in os.walk(dir_name):
                for filename in files:
                    from_path = os.path.join(dirpath, filename)
                    if ".DS_Store" in from_path or '.status' in from_path or '.git' in from_path:
                        continue
                    print(from_path)
                    to_path = f'{from_path}.temp'
                    file_encrypt(from_path, to_path, password, **kwargs)
                    os.remove(from_path)
                    copyfile(to_path, from_path)
                    os.remove(to_path)
        except Exception as e:
            rmtree(dir_name)
            copytree(backup_dir, dir_name)
            rmtree(backup_dir)
            status = open(status_file, "w")
            status.write("decrypted\n")
            status.close()
            raise e
        rmtree(backup_dir)


def dir_decrypt(password, dir_name, **kwargs):
    status_file = f'{dir_name}/.status'
    state = "decrypted"
    try:
        status = open(status_file, "r")
        state = status.readline().strip()
        status.close()
    except:
        pass
    if state == "encrypted":
        backup_dir = f'{dir_name}.backup'
        copytree(dir_name, backup_dir)
        try:
            for dirpath, subdirs, files in os.walk(dir_name):
                for filename in files:
                    to_path = os.path.join(dirpath, filename)
                    if ".DS_Store" in to_path or '.status' in to_path or '.git' in to_path:
                        continue
                    print(to_path)
                    from_path = f'{to_path}.temp'
                    copyfile(to_path, from_path)
                    os.remove(to_path)
                    file_decrypt(from_path, to_path, password, **kwargs)
                    os.remove(from_path)
            status = open(status_file, "w")
            status.write("decrypted\n")
            status.close()
        except Exception as e:
            rmtree(dir_name)
            copytree(backup_dir, dir_name)
            rmtree(backup_dir)
            raise e
        rmtree(backup_dir)




import string
import random

import boto3
from get_docker_secret import get_docker_secret
import os
import aws_encryption_sdk
from aws_encryption_sdk import LocalCryptoMaterialsCache, CachingCryptoMaterialsManager
import base64


# Cache capacity (maximum number of entries) is required
MAX_CACHE_SIZE = 10
cache = LocalCryptoMaterialsCache(MAX_CACHE_SIZE)

MAX_ENTRY_AGE_SECONDS = 60.0*60.0
MAX_ENTRY_MESSAGES = 100

FAKE_ENCRYPTION = os.environ.get('AWS_FAKE_ENCRYPTION', 'False') == 'True'


class CachingCMM():
    '''
    Actual encrypting CMM for use via AWS in prod
    '''

    def __init__(self, region=get_docker_secret('AWS_REGION'), account_number=get_docker_secret('AWS_ACCOUNT_NUMBER'), key_id=get_docker_secret('AWS_KEY_ID')):
        key_arn = 'arn:aws:kms:{0}:{1}:key/{2}'.format(region, account_number, key_id)
        kms_kwargs = dict(key_ids=[key_arn])
        master_key_provider = aws_encryption_sdk.KMSMasterKeyProvider(**kms_kwargs)
        self.caching_cmm = CachingCryptoMaterialsManager(
            master_key_provider=master_key_provider,
            cache=cache,
            max_age=MAX_ENTRY_AGE_SECONDS,
            max_messages_encrypted=MAX_ENTRY_MESSAGES
        )

    def encrypt(self, source_plaintext, **kwargs):
        ciphertext, encryptor_header = self._encrypt(source_plaintext, **kwargs)
        return ciphertext

    def _encrypt(self, source_plaintext, **kwargs):
        """Encrypts and then decrypts a string using a KMS customer master key (CMK)

        :param str key_arn: Amazon Resource Name (ARN) of the KMS CMK
        (http://docs.aws.amazon.com/kms/latest/developerguide/viewing-keys.html)
        :param bytes source_plaintext: Data to encrypt
        :param botocore_session: Existing Botocore session instance
        :type botocore_session: botocore.session.Session
        """

        # Encrypt the plaintext source data
        return aws_encryption_sdk.encrypt(
            source=source_plaintext,
            materials_manager=self.caching_cmm,
            **kwargs
        )

    def decrypt(self, ciphertext, **kwargs):
        plaintext_bytes, decrypted_header = self._decrypt(ciphertext, **kwargs)
        return plaintext_bytes.decode("utf-8")

    def _decrypt(self, ciphertext, **kwargs):

        # Decrypt the ciphertext
        return aws_encryption_sdk.decrypt(
            source=ciphertext,
            materials_manager=self.caching_cmm,
            **kwargs
        )


class FakeCMM():
    '''
    Fake CMM for faking encryption / decryption in dev
    '''

    def encrypt(self, source_plaintext, **kwargs):
        return str.encode(source_plaintext)

    def decrypt(self, ciphertext, **kwargs):
        return ciphertext.decode()

if FAKE_ENCRYPTION:
    cmm = FakeCMM()
else:
    cmm = CachingCMM()

def encrypt(source_plaintext, **kwargs):
    return cmm.encrypt(source_plaintext, **kwargs)

def decrypt(ciphertext, **kwargs):
    return cmm.decrypt(ciphertext, **kwargs)



def _create_data_key(region=get_docker_secret('AWS_REGION'), account_number=get_docker_secret('AWS_ACCOUNT_NUMBER'), key_id=get_docker_secret('AWS_KEY_ID'), key_spec="AES_256"):
    """Generate a data key to use when encrypting and decrypting data"""

    key_arn = 'arn:aws:kms:{0}:{1}:key/{2}'.format(region, account_number, key_id)

    # Create data key
    kms_client = boto3.client("kms", region_name=region)
    response = kms_client.generate_data_key(KeyId=key_arn, KeySpec=key_spec)

    # Return the encrypted and plaintext data key
    return response["CiphertextBlob"], base64.b64encode(response["Plaintext"]).decode('utf-8')

def _create_data_key_fake():
    letters = string.ascii_lowercase
    key = ''.join(random.choice(letters) for i in range(24))
    return bytes(key, 'utf-8'), key



if FAKE_ENCRYPTION:
    create_data_key = _create_data_key_fake
else:
    create_data_key = _create_data_key